#!/usr/bin/perl -wT
#/*
# Copyright (C) 2002 MandrakeSoft SA
# 
# This program may be distributed and/or
# modified under the terms of the GNU General Public
# License version 2 as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# Licensees holding valid MandrakeSecurity MNF Commercial License may use this
# file in  accordance with the MandrakeSecurity MNF Commercial License Agreement
# provided with the Software.
# 
# See http://www.mandrakesoft.com/mnf/license for information about MandrakeSecurity MNF Commercial License Agrrement.
#*/ 

#----------------------------------------------------------------------
# copyright (C) 1999, 2000 e-smith, inc.
# copyright (C) 2001 Mandrakesoft
# 		
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 		
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 		
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
# 
# Technical support for this program is available from e-smith, inc.
# Please visit our web site www.e-smith.net for details.
#----------------------------------------------------------------------

package backend::config;
use strict;

#------------------------------------------------------------
# The backend::config package enables Perl programs to read
# and write entries from the naat configuration file
# using a tied hash table.
# 
# The configuration file has a simple ASCII representation,
# with one "key=value" entry per line.
# 
# Default config file is /var/lib/naat/configuration.
#
# Usage example:
# 
# #!/usr/bin/perl -wT
# 
# use lib qw(/usr/lib/naat);
# use backend::config;
# use strict;
# 
# my %conf;
# tie %conf, 'backend::config', '/var/lib/naat/testconfig';
# 
# # write value to config file
# $conf {'DomainName'} = 'mycompany.xxx';
# 
# # read value from config file
# print $conf {'DomainName'} . "\n";
# 
# # dump contents of config file
# while (($key,$value) = each %conf)
# {
#     print "$key=$value\n";
# }
#------------------------------------------------------------

BEGIN
{
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
    $VERSION     = 1.00;
    @ISA         = qw(Exporter);
    @EXPORT      = ();
    @EXPORT_OK   = ();
    %EXPORT_TAGS = (); 
}

#------------------------------------------------------------
# readconf is a private subroutine.
# 
# It takes two arguments: the name of the config file and a
# reference to a hash.
# 
# The hash is cleared and then filled with key->value
# mappings read from the file.
#------------------------------------------------------------

sub readconf ($$)
{
    my ($filename, $confref) = @_;

    %$confref = ();

    # If the file cannot be opened, assume it hasn't been
    # created yet and return with an empty hash table.

    if (! open (FH, $filename))
    {
	return;
    }

    while (<FH>)
    {
	chomp;
	unless ( /=/ ) { next; }; # (Renaud) skip empty or erroneous lines
	my ($key, $value) = split (/=/, $_, 2);
	$$confref {$key} = $value;
    }

    close (FH);
    return;
}

#------------------------------------------------------------
# writeconf is a private subroutine.
# 
# It takes two arguments: the name of the config file and a
# reference to a hash.
# 
# The file is overwritten with the key->value mappings from
# the hash.
#------------------------------------------------------------

sub writeconf ($$)
{
    my ($filename, $confref) = @_;

    open (FH, ">$filename") || die "Cannot open $filename: $!\n";
    foreach (sort keys %$confref)
    {
	print FH $_ . "=" . $$confref{$_} . "\n";
    }

    close (FH);
    return;
}

#------------------------------------------------------------
# Constructor for the tied hash. If filename not specified,
# defaults to '/var/lib/naat/configuration'.
#------------------------------------------------------------

sub TIEHASH
{
    my $self = shift;
    my $filename = shift || '/var/lib/naat/configuration';

    my $node =
    {
	FILENAME => $filename,
	ENVCACHE => {},
    };

    readconf ($filename, $node->{'ENVCACHE'});

    return bless $node, $self;
}

#------------------------------------------------------------
# Look up a configuration parameter.
#------------------------------------------------------------

sub FETCH
{
    my $self = shift;
    my $key  = shift;

    return $self->{ENVCACHE} {$key};
}

#------------------------------------------------------------
# Store a configuration parameter.
#------------------------------------------------------------

sub STORE
{
    my $self  = shift;
    my $key   = shift;
    my $value = shift;

    $self->{ENVCACHE} {$key} = $value;

    # The 'UnsavedChanges' entry is automatically set to 'yes' when a
    # system parameter is changed. This means that there are changes
    # to the main e-smith configuration file which need to be 'saved'
    # (i.e.  all of the e-smith config files must be updated). However,
    # don't do anything automatic if the caller is deliberately trying
    # to set the UnsavedChanges flag. (That's how they can reset it.)

    if (($self->{FILENAME} eq '/home/e-smith/configuration') && ($key ne 'UnsavedChanges'))
    {
	$self->{ENVCACHE} {'UnsavedChanges'} = 'yes';
    }

    writeconf ($self->{FILENAME}, $self->{ENVCACHE});

    return undef;
}

#------------------------------------------------------------
# Delete a configuration parameter.
#------------------------------------------------------------

sub DELETE
{
    my $self = shift;
    my $key = shift;

    my $previous = delete $self->{ENVCACHE} {$key};
    writeconf ($self->{FILENAME}, $self->{ENVCACHE});

    return $previous;
}

#------------------------------------------------------------
# Clear the configuration file.
#------------------------------------------------------------

sub CLEAR
{
    my $self = shift;

    $self->{ENVCACHE} = ();
    writeconf ($self->{FILENAME}, $self->{ENVCACHE});

    return undef;
}

#------------------------------------------------------------
# Check whether a particular key exists in the configuration file.
#------------------------------------------------------------

sub EXISTS
{
    my $self = shift;
    my $key = shift;

    return exists $self->{ENVCACHE} {$key};
}

#------------------------------------------------------------
# FIRSTKEY is called whenever we start iterating over the
# configuration table. We cache the configuration table at
# this point to ensure reasonable results if the
# configuration file is changed by another program during
# the iteration.
#------------------------------------------------------------

sub FIRSTKEY
{
    my $self = shift;

    my $discard = keys %{$self->{ENVCACHE}};    # reset each() iterator

    return each %{$self->{ENVCACHE}};
}

#------------------------------------------------------------
# NEXTKEY is called for all iterations after the first.  We
# just keep returning results from the cached configuration
# table.  A null array is returned at the end. If the caller
# starts a new iteration, the FIRSTKEY subroutine is called
# again, causing the cache to be reloaded.
#------------------------------------------------------------

sub NEXTKEY
{
    my $self = shift;
    return each %{$self->{ENVCACHE}};
}

#------------------------------------------------------------
# No special instructions for the destructor.
#------------------------------------------------------------

sub DESTROY
{
}

#------------------------------------------------------------
# No module clean-up code required.
#------------------------------------------------------------

END
{
}

#------------------------------------------------------------
# Return "1" to make the import process return success.
#------------------------------------------------------------

1;
